Package: oce
Type: Package
Title: Analysis of Oceanographic Data
Version: 1.0-1
Authors@R: c(
    person(given="Dan", family="Kelley", email="Dan.Kelley@Dal.Ca", role=c("aut", "cre"), comment=c(ORCID="https://orcid.org/0000-0001-7808-5911")),
    person(given="Clark", family="Richards", email="clark.richards@gmail.com", role=c("aut"), comment=c(ORCID="https://orcid.org/0000-0002-7833-206X")),
    person(given="Chantelle", family="Layton", email="chantelle.layton@dal.ca", role=c("ctb"), comment=c(ORCID="https://orcid.org/0000-0002-3199-5763", "curl() coauthor")),
    person(given="British Geological Survey", role=c("ctb","cph"), comment="magnetic-field subroutine"))
Maintainer: Dan Kelley <Dan.Kelley@Dal.Ca>
Depends: R (>= 2.15), utils, methods, testthat, gsw
Suggests: ocedata, foreign, ncdf4, tiff, akima, RSQLite, DBI, rgdal (>=
        1.1-9), R.utils, knitr, rmarkdown, marmap, lubridate
BugReports: https://github.com/dankelley/oce/issues
Description: Supports the analysis of Oceanographic data, including 'ADCP'
    measurements, measurements made with 'argo' floats, 'CTD' measurements,
    sectional data, sea-level time series, coastline and topographic data, etc.
    Provides specialized functions for calculating seawater properties such as
    potential temperature in either the 'UNESCO' or 'TEOS-10' equation of state.
    Produces graphical displays that conform to the conventions of the Oceanographic
    literature.
License: GPL (>= 2)
Encoding: UTF-8
URL: https://dankelley.github.io/oce
LazyData: false
RoxygenNote: 6.1.0
BuildVignettes: true
VignetteBuilder: knitr
NeedsCompilation: yes
LinkingTo: Rcpp
Imports: Rcpp
Packaged: 2018-10-04 15:11:17 UTC; kelley
Author: Dan Kelley [aut, cre] (<https://orcid.org/0000-0001-7808-5911>),
  Clark Richards [aut] (<https://orcid.org/0000-0002-7833-206X>),
  Chantelle Layton [ctb] (<https://orcid.org/0000-0002-3199-5763>, curl()
    coauthor),
  British Geological Survey [ctb, cph] (magnetic-field subroutine)
Repository: CRAN
Date/Publication: 2018-10-04 16:00:03 UTC
Built: R 3.5.1; x86_64-w64-mingw32; 2018-10-08 01:05:50 UTC; windows
Archs: x64
